@@ -37,13 +37,17 @@ def initialize(
     api_host=None,  # type: Optional[str]
     statsd_host=None,  # type: Optional[str]
     statsd_port=None,  # type: Optional[int]
+    statsd_disable_aggregation=True,  # type: bool
     statsd_disable_buffering=True,  # type: bool
+    statsd_aggregation_flush_interval=0.3,  # type: float
     statsd_use_default_route=False,  # type: bool
     statsd_socket_path=None,  # type: Optional[str]
     statsd_namespace=None,  # type: Optional[str]
+    statsd_max_samples_per_context=0,  # type: Optional[int]
     statsd_constant_tags=None,  # type: Optional[List[str]]
     return_raw_response=False,  # type: bool
     hostname_from_config=True,  # type: bool
+    cardinality=None,  # type: Optional[str]
     **kwargs  # type: Any
 ):
     # type: (...) -> None
@@ -76,6 +80,19 @@ def initialize(
                                      (default: True).
     :type statsd_disable_buffering: boolean
 
+    :param statsd_disable_aggregation: Enable/disable statsd client aggregation support
+                                     (default: True).
+    :type statsd_disable_aggregation: boolean
+
+    :param statsd_max_samples_per_context: Set the max samples per context for Histogram,
+    Distribution and Timing metrics. Use with the statsd_disable_aggregation set to False.
+    :type statsd_max_samples_per_context: int
+
+    :param statsd_aggregation_flush_interval: If aggregation is enabled, set the flush interval for
+                    aggregation/buffering (This feature is experimental)
+                                     (default: 0.3 seconds)
+    :type statsd_aggregation_flush_interval: float
+
     :param statsd_use_default_route: Dynamically set the statsd host to the default route
                                      (Useful when running the client in a container)
     :type statsd_use_default_route: boolean
@@ -101,6 +118,12 @@ def initialize(
 
     :param hostname_from_config: Set the hostname from the Datadog agent config (agent 5). Will be deprecated
     :type hostname_from_config: boolean
+
+    :param cardinality: Set the global cardinality for all metrics. \
+        Possible values are "none", "low", "orchestrator" and "high".
+        Can also be set via the DATADOG_CARDINALITY or DD_CARDINALITY environment variables.
+    :type cardinality: string
+
     """
     # API configuration
     api._api_key = api_key or api._api_key or os.environ.get("DATADOG_API_KEY", os.environ.get("DD_API_KEY"))
@@ -122,15 +145,22 @@ def initialize(
             statsd.host = statsd.resolve_host(statsd_host, statsd_use_default_route)
         if statsd_port:
             statsd.port = int(statsd_port)
+    statsd.close_socket()
     if statsd_namespace:
         statsd.namespace = text(statsd_namespace)
     if statsd_constant_tags:
         statsd.constant_tags += statsd_constant_tags
 
+    if statsd_disable_aggregation:
+        statsd.disable_aggregation()
+    else:
+        statsd.enable_aggregation(statsd_aggregation_flush_interval, statsd_max_samples_per_context)
     statsd.disable_buffering = statsd_disable_buffering
-
     api._return_raw_response = return_raw_response
 
+    # Set the global cardinality for all metrics
+    statsd.cardinality = cardinality or os.environ.get("DATADOG_CARDINALITY", os.environ.get("DD_CARDINALITY"))
+
     # HTTP client and API options
     for key, value in iteritems(kwargs):
         attribute = "_{}".format(key)
