@@ -8,4 +8,5 @@ class Comment(CreateableAPIResource, UpdatableAPIResource):
     """
     A wrapper around Comment HTTP API.
     """
-    _resource_name = 'comments'
+
+    _resource_name = "comments"
