@@ -17,6 +17,7 @@ from datadog.dogshell.dashboard_list import DashboardListClient
 from datadog.dogshell.downtime import DowntimeClient
 from datadog.dogshell.event import EventClient
 from datadog.dogshell.host import HostClient
+from datadog.dogshell.hosts import HostsClient
 from datadog.dogshell.metric import MetricClient
 from datadog.dogshell.monitor import MonitorClient
 from datadog.dogshell.screenboard import ScreenboardClient
@@ -26,6 +27,7 @@ from datadog.dogshell.service_level_objective import ServiceLevelObjectiveClient
 from datadog.dogshell.tag import TagClient
 from datadog.dogshell.timeboard import TimeboardClient
 from datadog.dogshell.dashboard import DashboardClient
+from datadog.dogshell.security_monitoring import SecurityMonitoringClient
 
 
 def main():
@@ -73,9 +75,9 @@ def main():
 
     parser.add_argument(
         "--api_host",
-        help="Datadog site to send data, us (datadoghq.com), eu (datadoghq.eu) or us3 (us3.datadoghq.com), default: us",
+        help="Datadog site to send data, us (datadoghq.com), eu (datadoghq.eu), us3 (us3.datadoghq.com), \
+              us5 (us5.datadoghq.com), ap1 (ap1.datadoghq.com), gov (ddog-gov.com), or custom url. default: us",
         dest="api_host",
-        choices=["datadoghq.com", "us", "datadoghq.eu", "eu", "us3.datadoghq.com", "us3"],
     )
 
     config = DogshellConfig()
@@ -95,9 +97,11 @@ def main():
     ScreenboardClient.setup_parser(subparsers)
     DashboardListClient.setup_parser(subparsers)
     HostClient.setup_parser(subparsers)
+    HostsClient.setup_parser(subparsers)
     DowntimeClient.setup_parser(subparsers)
     ServiceCheckClient.setup_parser(subparsers)
     ServiceLevelObjectiveClient.setup_parser(subparsers)
+    SecurityMonitoringClient.setup_parser(subparsers)
 
     args = parser.parse_args()
 
