@@ -1,6 +1,7 @@
 # Unless explicitly stated otherwise all files in this repository are licensed under the BSD-3-Clause License.
 # This product includes software developed at Datadog (https://www.datadoghq.com/).
 # Copyright 2015-Present Datadog, Inc
+import datetime
 from hashlib import md5
 import json
 import os
@@ -399,11 +400,14 @@ class TestDogshell:
         # Create a monitor
         query = "avg(last_1h):sum:system.net.bytes_rcvd{*} by {host} > 100"
         type_alert = "metric alert"
-        out, _, _ = dogshell(["monitor", "post", type_alert, query])
+        tags = "main,test"
+        priority = "5"
+        out, _, _ = dogshell(["monitor", "post", type_alert, query, "--tags", tags, "--priority", priority])
 
         out = json.loads(out)
         assert out["query"] == query
         assert out["type"] == type_alert
+        assert out["priority"] == int(priority)
         monitor_id = str(out["id"])
         monitor_name = out["name"]
 
@@ -439,12 +443,15 @@ class TestDogshell:
         assert monitor_name == out["name"]
         assert current_options == out["options"]
 
-        # Updating optional type and query
+        # Updating optional type, query, tags, and priority
         updated_query = "avg(last_15m):sum:system.net.bytes_rcvd{*} by {env} > 222"
         updated_type = "query alert"
+        updated_tags = "main"
+        updated_priority = "4"
 
         out, err, return_code = dogshell(
-            ["monitor", "update", monitor_id, "--type", updated_type, "--query", updated_query]
+            ["monitor", "update", monitor_id, "--type", updated_type, "--query", updated_query,
+             "--tags", updated_tags, "--priority", updated_priority]
         )
 
         out = json.loads(out)
@@ -453,6 +460,7 @@ class TestDogshell:
         assert updated_message in out["message"] # updated_message updated in previous step
         assert monitor_name in out["name"]
         assert current_options == out["options"]
+        assert int(updated_priority) == out["priority"]
 
         # Mute monitor
         out, _, _ = dogshell(["monitor", "mute", str(out["id"])])
@@ -609,6 +617,40 @@ class TestDogshell:
         assert out["action"] == "Unmuted"
         assert out["hostname"] == hostname
 
+    def test_hosts_list(self, dogshell):
+        # `dog hosts list` should return a list of hosts
+        params = {
+            "filter": "env",
+            "sort_field": "host_name",
+            "sort_dir": "asc",
+            "start": 0,
+            "count": 100,
+            "from_": 0,
+            "include_muted_hosts_data": True,
+            "include_hosts_metadata": True,
+        }
+        
+        out, _, _ = dogshell(["hosts", "list",
+                                        "--filter", params["filter"],
+                                        "--sort_field", params["sort_field"],
+                                        "--sort_dir", params["sort_dir"], 
+                                        "--start", str(params["start"]),
+                                        "--count", str(params["count"]), 
+                                        "--from", str(params["from_"])],
+                                        "--include_muted_hosts_data",
+                                        "--include_hosts_metadata",
+                                        )
+
+        out = json.loads(out)
+        assert out["host_list"] is not None
+        assert out["total_matching"] >= 1
+
+    def test_hosts_totals(self, dogshell):
+        # `dog hosts totals` should return the total number of hosts
+        out, _, _ = dogshell(["hosts", "totals"])
+        out = json.loads(out)
+        assert out["total_active"] >= 1
+
     def test_downtime_schedule(self, freezer, dogshell):
         # Schedule a downtime
         scope = "env:staging"
@@ -669,6 +711,123 @@ class TestDogshell:
         out, _, _ = dogshell(["service_check", "check", "check_pg", "host0", "1"])
         out = json.loads(out)
         assert out["status"], "ok"
+        
+    def test_security_monitoring_rules(self, dogshell, get_unique, tmp_path):
+        """
+        Test dogshell security-monitoring rules commands.
+        """
+        # Generate a unique rule name to avoid conflicts
+        unique = get_unique()
+        rule_name = "Test Rule {}".format(unique)
+
+        # Create a rule file
+        rule_file = tmp_path / "rule.json"
+        rule_data = {
+            "name": rule_name,
+            "queries": [
+                {
+                    "query": "source:test",
+                    "groupByFields": [],
+                    "distinctFields": [],
+                    "name": "a"
+                }
+            ],
+            "cases": [
+                {
+                    "name": "test case",
+                    "condition": "a > 1",
+                    "status": "info"
+                }
+            ],
+            "options": {
+                "evaluationWindow": 3600,
+                "keepAlive": 3600,
+                "maxSignalDuration": 86400
+            },
+            "message": "Test rule generated via dogshell",
+            "tags": ["test:true", "env:frog"],
+            "isEnabled": True,
+            "type": "log_detection"
+        }
+        
+        with open(rule_file, "w") as f:
+            json.dump(rule_data, f)
+            
+        # Create rule
+        out, _, _ = dogshell(["security-monitoring", "rules", "create", "--file", str(rule_file)])
+        created_rule = json.loads(out)
+        assert created_rule["name"] == rule_name
+        assert "id" in created_rule
+        rule_id = created_rule["id"]
+        
+        # Get rule
+        out, _, _ = dogshell(["security-monitoring", "rules", "get", rule_id])
+        retrieved_rule = json.loads(out)
+        assert retrieved_rule["id"] == rule_id
+        assert retrieved_rule["name"] == rule_name
+        
+        # Skip the list rule since there is not filtering
+        
+        # Update rule
+        updated_name = "Updated Rule {}".format(unique)
+        rule_data["name"] = updated_name
+        
+        with open(rule_file, "w") as f:
+            json.dump(rule_data, f)
+            
+        out, _, _ = dogshell(["security-monitoring", "rules", "update", rule_id, "--file", str(rule_file)])
+        updated_rule = json.loads(out)
+        assert updated_rule["id"] == rule_id
+        assert updated_rule["name"] == updated_name
+        
+        # Delete rule
+        out, _, _ = dogshell(["security-monitoring", "rules", "delete", rule_id])
+        # Successfully deleted returns a valid response
+        assert out == ""
+        
+    def test_security_monitoring_signals(self, freezer, dogshell):
+        """
+        Test dogshell security-monitoring signals commands.
+        Note: Since signals cannot be created directly, we'll primarily test command structure.
+        """
+        with freezer:
+            now_time = datetime.datetime.now(datetime.timezone.utc)
+            now = now_time.replace(microsecond=0).isoformat()
+            one_hour_ago = (now_time - datetime.timedelta(hours=1)).replace(microsecond=0).isoformat()
+
+        # List signals
+        out, err, return_code = dogshell(
+            [
+                "security-monitoring", 
+                "signals", 
+                "list", 
+                "--query", "security:attack",
+                "--from", one_hour_ago,
+                "--to", now,
+                "--sort", "-timestamp",
+                "--page-size", "5"
+            ],
+            check_return_code=False
+        )
+        
+        # Even if no signals are found, the command should execute successfully
+        # If it's a 401/403, that's okay since test account might not have permissions
+        if return_code == 0:
+            signals = json.loads(out)
+            assert "data" in signals or "signals" in signals
+        
+        # Try get signal with a fake ID (will likely fail, but tests command structure)
+        fake_signal_id = "abcd1234-abcd-1234-abcd-1234abcd1234"
+        _, _, return_code = dogshell(
+            ["security-monitoring", "signals", "get", fake_signal_id],
+            check_return_code=False
+        )
+        
+        # Try the triage command with a fake ID (will likely fail, but tests command structure)
+        _, _, return_code = dogshell(
+            ["security-monitoring", "signals", "triage", fake_signal_id, "--state", "archived"],
+            check_return_code=False
+        )
 
     def parse_response(self, out):
         data = {}
