@@ -10,6 +10,7 @@ Tests for dogstatsd.py
 # Standard libraries
 from collections import deque
 from contextlib import closing
+import struct
 from threading import Thread
 import errno
 import os
@@ -29,7 +30,7 @@ import pytest
 # Datadog libraries
 from datadog import initialize, statsd
 from datadog import __version__ as version
-from datadog.dogstatsd.base import DEFAULT_FLUSH_INTERVAL, DogStatsd, MIN_SEND_BUFFER_SIZE, UDP_OPTIMAL_PAYLOAD_LENGTH, UDS_OPTIMAL_PAYLOAD_LENGTH
+from datadog.dogstatsd.base import DEFAULT_BUFFERING_FLUSH_INTERVAL, DogStatsd, MIN_SEND_BUFFER_SIZE, UDP_OPTIMAL_PAYLOAD_LENGTH, UDS_OPTIMAL_PAYLOAD_LENGTH
 from datadog.dogstatsd.context import TimedContextManagerDecorator
 from datadog.util.compat import is_higher_py35, is_p3k
 from tests.util.contextmanagers import preserve_environment_variable, EnvVars
@@ -41,11 +42,21 @@ class FakeSocket(object):
 
     FLUSH_GRACE_PERIOD = 0.2
 
-    def __init__(self, flush_interval=DEFAULT_FLUSH_INTERVAL):
+    def __init__(self, flush_interval=DEFAULT_BUFFERING_FLUSH_INTERVAL, socket_kind=socket.SOCK_DGRAM, socket_path=None):
         self.payloads = deque()
 
         self._flush_interval = flush_interval
         self._flush_wait = False
+        self._socket_kind = socket_kind
+        self.timeout = () # unit tuple = settimeout was not called
+
+        if socket_path:
+            self.family = socket.AF_UNIX
+        else:
+            self.family = socket.AF_INET
+
+    def sendall(self, payload):
+        self.send(payload)
 
     def send(self, payload):
         if is_p3k():
@@ -63,20 +74,34 @@ class FakeSocket(object):
                 time.sleep(self._flush_interval+self.FLUSH_GRACE_PERIOD)
             self._flush_wait = True
 
-        if count > len(self.payloads):
+        payload_len = len(self.payloads)
+        if self._socket_kind == socket.SOCK_STREAM:
+            if payload_len % 2 != 0 or count > (payload_len / 2):
+                return None
+        elif count > len(self.payloads):
             return None
 
         out = []
         for _ in range(count):
-            out.append(self.payloads.popleft().decode('utf-8'))
+            if self._socket_kind == socket.SOCK_DGRAM:
+                out.append(self.payloads.popleft().decode('utf-8'))
+            else:
+                length = struct.unpack('<I', self.payloads.popleft())[0]
+                pl = self.payloads.popleft()[:length].decode('utf-8')
+                out.append(pl)
         return '\n'.join(out)
 
     def close(self):
         pass
 
+    def getsockopt(self, *args):
+        return self._socket_kind
+
     def __repr__(self):
         return str(self.payloads)
 
+    def settimeout(self, timeout):
+        self.timeout = timeout
 
 class BrokenSocket(FakeSocket):
     def __init__(self, error_number=None):
@@ -98,7 +123,7 @@ class OverflownSocket(BrokenSocket):
         super(OverflownSocket, self).__init__(errno.EAGAIN)
 
 
-def telemetry_metrics(metrics=1, events=0, service_checks=0, bytes_sent=0, bytes_dropped=0, packets_sent=1, packets_dropped=0, transport="udp", tags=""):
+def telemetry_metrics(metrics=1, events=0, service_checks=0, bytes_sent=0, bytes_dropped_writer=0, packets_sent=1, packets_dropped_writer=0, transport="udp", tags="", bytes_dropped_queue=0, packets_dropped_queue=0):
     tags = "," + tags if tags else ""
 
     return "\n".join([
@@ -106,9 +131,13 @@ def telemetry_metrics(metrics=1, events=0, service_checks=0, bytes_sent=0, bytes
         "datadog.dogstatsd.client.events:{}|c|#client:py,client_version:{},client_transport:{}{}".format(events, version, transport, tags),
         "datadog.dogstatsd.client.service_checks:{}|c|#client:py,client_version:{},client_transport:{}{}".format(service_checks, version, transport, tags),
         "datadog.dogstatsd.client.bytes_sent:{}|c|#client:py,client_version:{},client_transport:{}{}".format(bytes_sent, version, transport, tags),
-        "datadog.dogstatsd.client.bytes_dropped:{}|c|#client:py,client_version:{},client_transport:{}{}".format(bytes_dropped, version, transport, tags),
+        "datadog.dogstatsd.client.bytes_dropped:{}|c|#client:py,client_version:{},client_transport:{}{}".format(bytes_dropped_queue + bytes_dropped_writer, version, transport, tags),
+        "datadog.dogstatsd.client.bytes_dropped_queue:{}|c|#client:py,client_version:{},client_transport:{}{}".format(bytes_dropped_queue, version, transport, tags),
+        "datadog.dogstatsd.client.bytes_dropped_writer:{}|c|#client:py,client_version:{},client_transport:{}{}".format(bytes_dropped_writer, version, transport, tags),
         "datadog.dogstatsd.client.packets_sent:{}|c|#client:py,client_version:{},client_transport:{}{}".format(packets_sent, version, transport, tags),
-        "datadog.dogstatsd.client.packets_dropped:{}|c|#client:py,client_version:{},client_transport:{}{}".format(packets_dropped, version, transport, tags)
+        "datadog.dogstatsd.client.packets_dropped:{}|c|#client:py,client_version:{},client_transport:{}{}".format(packets_dropped_queue + packets_dropped_writer, version, transport, tags),
+        "datadog.dogstatsd.client.packets_dropped_queue:{}|c|#client:py,client_version:{},client_transport:{}{}".format(packets_dropped_queue, version, transport, tags),
+        "datadog.dogstatsd.client.packets_dropped_writer:{}|c|#client:py,client_version:{},client_transport:{}{}".format(packets_dropped_writer, version, transport, tags),
     ]) + "\n"
 
 
@@ -138,15 +167,16 @@ class TestDogStatsd(unittest.TestCase):
         """
         self._procfs_mock.stop()
 
-    def assert_equal_telemetry(self, expected_payload, actual_payload, telemetry=None):
+    def assert_equal_telemetry(self, expected_payload, actual_payload, telemetry=None, **kwargs):
         if telemetry is None:
-            telemetry = telemetry_metrics(bytes_sent=len(expected_payload))
+            telemetry = telemetry_metrics(bytes_sent=len(expected_payload), **kwargs)
 
         if expected_payload:
             expected_payload = "\n".join([expected_payload, telemetry])
         else:
             expected_payload = telemetry
 
+        self.maxDiff = None
         return self.assertEqual(expected_payload, actual_payload)
 
     def send_and_assert(
@@ -261,6 +291,11 @@ class TestDogStatsd(unittest.TestCase):
         self.assertIsNone(statsd.host)
         self.assertIsNone(statsd.port)
 
+        # Add cardinality
+        options['cardinality'] = 'none'
+        initialize(**options)
+        self.assertEqual(statsd.cardinality, 'none')
+
     def test_dogstatsd_initialization_with_env_vars(self):
         """
         Dogstatsd can retrieve its config from env vars when
@@ -277,6 +312,12 @@ class TestDogStatsd(unittest.TestCase):
         self.assertEqual(dogstatsd.host, "myenvvarhost")
         self.assertEqual(dogstatsd.port, 4321)
 
+    def test_initialization_closes_socket(self):
+        statsd.socket = FakeSocket()
+        self.assertIsNotNone(statsd.socket)
+        initialize()
+        self.assertIsNone(statsd.socket)
+
     def test_default_route(self):
         """
         Dogstatsd host can be dynamically set to the default route.
@@ -290,10 +331,43 @@ class TestDogStatsd(unittest.TestCase):
         self.statsd.set('set', 123)
         self.assert_equal_telemetry('set:123|s\n', self.recv(2))
 
+    def test_report(self):
+        self.statsd._report('report', 'g', 123.4, tags=None, sample_rate=None)
+        self.assert_equal_telemetry('report:123.4|g\n', self.recv(2))
+
+    def test_report_metric_with_unsupported_ts(self):
+        self.statsd._reset_telemetry()
+        self.statsd._report('report', 'h', 123.5, tags=None, sample_rate=None, timestamp=100)
+        self.assert_equal_telemetry('report:123.5|h\n', self.recv(2))
+
+        self.statsd._reset_telemetry()
+        self.statsd._report('set', 's', 123, tags=None, sample_rate=None, timestamp=100)
+        self.assert_equal_telemetry('set:123|s\n', self.recv(2))
+
+    def test_report_with_cardinality(self):
+        self.statsd._report('report', 'g', 123.4, tags=None, sample_rate=None, cardinality="orchestrator")
+        self.assert_equal_telemetry('report:123.4|g|card:orchestrator\n', self.recv(2))
+
     def test_gauge(self):
         self.statsd.gauge('gauge', 123.4)
         self.assert_equal_telemetry('gauge:123.4|g\n', self.recv(2))
 
+    def test_gauge_with_ts(self):
+        self.statsd.gauge_with_timestamp("gauge", 123.4, timestamp=1066)
+        self.assert_equal_telemetry("gauge:123.4|g|T1066\n", self.recv(2))
+
+    def test_gauge_with_cardinality(self):
+        self.statsd.gauge('gauge', 123.4, cardinality="high")
+        self.assert_equal_telemetry('gauge:123.4|g|card:high\n', self.recv(2))
+
+        self.statsd._reset_telemetry()
+        self.statsd.gauge_with_timestamp("gauge", 123.4, timestamp=1066, cardinality="none")
+        self.assert_equal_telemetry("gauge:123.4|g|card:none|T1066\n", self.recv(2))
+
+    def test_gauge_with_invalid_ts_should_be_ignored(self):
+        self.statsd.gauge_with_timestamp("gauge", 123.4, timestamp=-500)
+        self.assert_equal_telemetry("gauge:123.4|g\n", self.recv(2))
+
     def test_counter(self):
         self.statsd.increment('page.views')
         self.statsd.flush()
@@ -314,10 +388,44 @@ class TestDogStatsd(unittest.TestCase):
         self.statsd.flush()
         self.assert_equal_telemetry('page.views:-12|c\n', self.recv(2))
 
+    def test_count(self):
+        self.statsd.count('page.views', 11)
+        self.statsd.flush()
+        self.assert_equal_telemetry('page.views:11|c\n', self.recv(2))
+
+    def test_count_with_ts(self):
+        self.statsd.count_with_timestamp("page.views", 1, timestamp=1066)
+        self.statsd.flush()
+        self.assert_equal_telemetry("page.views:1|c|T1066\n", self.recv(2))
+
+        self.statsd._reset_telemetry()
+        self.statsd.count_with_timestamp("page.views", 11, timestamp=2121)
+        self.statsd.flush()
+        self.assert_equal_telemetry("page.views:11|c|T2121\n", self.recv(2))
+
+    def test_count_with_cardinality(self):
+        self.statsd.count('page.views', 11, cardinality="low")
+        self.statsd.flush()
+        self.assert_equal_telemetry('page.views:11|c|card:low\n', self.recv(2))
+
+        self.statsd._reset_telemetry()
+        self.statsd.count_with_timestamp("page.views", 11, timestamp=2121, cardinality="high")
+        self.statsd.flush()
+        self.assert_equal_telemetry("page.views:11|c|card:high|T2121\n", self.recv(2))
+
+    def test_count_with_invalid_ts_should_be_ignored(self):
+        self.statsd.count_with_timestamp("page.views", 1, timestamp=-1066)
+        self.statsd.flush()
+        self.assert_equal_telemetry("page.views:1|c\n", self.recv(2))
+
     def test_histogram(self):
         self.statsd.histogram('histo', 123.4)
         self.assert_equal_telemetry('histo:123.4|h\n', self.recv(2))
 
+    def test_histogram_with_cardinality(self):
+        self.statsd.histogram('histo', 123.4, cardinality="low")
+        self.assert_equal_telemetry('histo:123.4|h|card:low\n', self.recv(2))
+
     def test_pipe_in_tags(self):
         self.statsd.gauge('gt', 123.4, tags=['pipe|in:tag', 'red'])
         self.assert_equal_telemetry('gt:123.4|g|#pipe_in:tag,red\n', self.recv(2))
@@ -396,8 +504,9 @@ class TestDogStatsd(unittest.TestCase):
             u'L1\nL2',
             priority='low',
             date_happened=1375296969,
+            cardinality="orchestrator",
         )
-        event2 = u'_e{5,6}:Title|L1\\nL2|d:1375296969|p:low\n'
+        event2 = u'_e{5,6}:Title|L1\\nL2|d:1375296969|p:low|card:orchestrator\n'
         self.assert_equal_telemetry(
             event2,
             self.recv(2),
@@ -441,6 +550,22 @@ class TestDogStatsd(unittest.TestCase):
 
         self.statsd._reset_telemetry()
 
+    # Positional arg names should match threadstats
+    def test_event_matching_signature(self):
+        self.statsd.event(title="foo", message="bar1")
+        event = u'_e{3,4}:foo|bar1\n'
+        self.assert_equal_telemetry(
+            event,
+            self.recv(2),
+            telemetry=telemetry_metrics(
+                metrics=0,
+                events=1,
+                bytes_sent=len(event),
+            ),
+        )
+
+        self.statsd._reset_telemetry()
+
     def test_event_constant_tags(self):
         self.statsd.constant_tags = ['bar:baz', 'foo']
         self.statsd.event('Title', u'L1\nL2', priority='low', date_happened=1375296969)
@@ -472,13 +597,31 @@ class TestDogStatsd(unittest.TestCase):
             ),
         )
 
+    def test_event_payload_error(self):
+        def func():
+            # define an event payload that is > 8 * 1024
+            message = ["l" for i in range(8 * 1024)]
+            message = "".join(message)
+            payload = {"title": "title", "message": message}
+
+            self.statsd.event(**payload)
+
+        # check that the method fails when the payload is too large
+        with pytest.raises(ValueError):
+            func()
+
+        # check that the method does not fail with a small payload
+        self.statsd.event("title", "message")
+
     def test_service_check(self):
         now = int(time.time())
         self.statsd.service_check(
             'my_check.name', self.statsd.WARNING,
             tags=['key1:val1', 'key2:val2'], timestamp=now,
-            hostname='i-abcd1234', message=u"♬ †øU \n†øU ¥ºu|m: T0µ ♪")
-        check = u'_sc|my_check.name|{0}|d:{1}|h:i-abcd1234|#key1:val1,key2:val2|m:{2}'.format(self.statsd.WARNING, now, u'♬ †øU \\n†øU ¥ºu|m\\: T0µ ♪\n')
+            hostname='i-abcd1234', message=u"♬ †øU \n†øU ¥ºu|m: T0µ ♪",
+            cardinality="low",
+        )
+        check = u'_sc|my_check.name|{0}|d:{1}|h:i-abcd1234|#key1:val1,key2:val2|m:{2}|card:low\n'.format(self.statsd.WARNING, now, u'♬ †øU \\n†øU ¥ºu|m\\: T0µ ♪')
         self.assert_equal_telemetry(
             check,
             self.recv(2),
@@ -534,7 +677,7 @@ class TestDogStatsd(unittest.TestCase):
         self.statsd.gauge('gauge', 123.4)
         self.assert_equal_telemetry('foo.gauge:123.4|g\n', self.recv(2))
 
-    # Test Client level contant tags
+    # Test Client level content tags
     def test_gauge_constant_tags(self):
         self.statsd.constant_tags = ['bar:baz', 'foo']
         self.statsd.gauge('gauge', 123.4)
@@ -656,7 +799,7 @@ class TestDogStatsd(unittest.TestCase):
         datadog.flush()
 
         # Sanity check
-        mock_socket_create.assert_called_once_with(socket.AF_INET, socket.SOCK_DGRAM)
+        mock_socket_create.assert_called_once_with(socket.AF_INET, socket.SOCK_DGRAM, socket.IPPROTO_UDP)
 
         mock_socket.setsockopt.assert_called_once_with(
             socket.SOL_SOCKET,
@@ -664,6 +807,23 @@ class TestDogStatsd(unittest.TestCase):
             MIN_SEND_BUFFER_SIZE,
         )
 
+    def test_socket_updates_telemetry(self):
+        # Test UDP
+        self.statsd.gauge("foo", 1)
+        self.assert_equal_telemetry("foo:1|g\n", self.recv(2), transport="udp")
+        
+        # Test UDS
+        self.statsd.socket = FakeSocket(socket_path="/fake/path")
+        self.statsd._reset_telemetry()
+        self.statsd.gauge("foo", 2)
+        self.assert_equal_telemetry("foo:2|g\n", self.recv(2), transport="uds")
+
+        # Test UDS stream
+        self.statsd.socket = FakeSocket(socket_path="unixstream://fake/path", socket_kind=socket.SOCK_STREAM)
+        self.statsd._reset_telemetry()
+        self.statsd.gauge("foo", 2)
+        self.assert_equal_telemetry("foo:2|g\n", self.recv(2), transport="uds-stream")
+
     def test_distributed(self):
         """
         Measure the distribution of a function's run time using distribution custom metric.
@@ -707,7 +867,7 @@ class TestDogStatsd(unittest.TestCase):
         self.assertEqual('d', type_)
         self.assertEqual('distributed.test', name)
         self.assert_almost_equal(500, float(value), 100)
-
+        
     def test_timed(self):
         """
         Measure the distribution of a function's run time.
@@ -966,22 +1126,34 @@ async def print_foo():
                 telemetry=telemetry_metrics(metrics=2, bytes_sent=len(expected))
         )
 
-    def test_flush(self):
+    def test_flush_dgram(self):
+        self._test_flush(socket.SOCK_DGRAM)
+
+    def test_flush_stream(self):
+        self._test_flush(socket.SOCK_STREAM)
+
+    def _test_flush(self, socket_kind):
         dogstatsd = DogStatsd(disable_buffering=False, telemetry_min_flush_interval=0)
-        fake_socket = FakeSocket()
+        fake_socket = FakeSocket(socket_kind=socket_kind)
         dogstatsd.socket = fake_socket
 
-        dogstatsd.increment('page.views')
+        dogstatsd.increment(u'page.®views®')
         self.assertIsNone(fake_socket.recv(no_wait=True))
         dogstatsd.flush()
-        self.assert_equal_telemetry('page.views:1|c\n', fake_socket.recv(2))
+        self.assert_equal_telemetry(u'page.®views®:1|c\n', fake_socket.recv(2))
 
-    def test_flush_interval(self):
+    def test_flush_interval_dgram(self):
+        self._test_flush_interval(socket.SOCK_DGRAM)
+
+    def test_flush_interval_stream(self):
+        self._test_flush_interval(socket.SOCK_STREAM)
+
+    def _test_flush_interval(self, socket_kind):
         dogstatsd = DogStatsd(disable_buffering=False, flush_interval=1, telemetry_min_flush_interval=0)
-        fake_socket = FakeSocket()
+        fake_socket = FakeSocket(socket_kind=socket_kind)
         dogstatsd.socket = fake_socket
 
-        dogstatsd.increment('page.views')
+        dogstatsd.increment(u'page.®views®')
         self.assertIsNone(fake_socket.recv(no_wait=True))
 
         time.sleep(0.3)
@@ -989,7 +1161,47 @@ async def print_foo():
 
         time.sleep(1)
         self.assert_equal_telemetry(
-            'page.views:1|c\n',
+            u'page.®views®:1|c\n',
+            fake_socket.recv(2, no_wait=True)
+        )
+    
+    def test_aggregation_buffering_simultaneously_dgram(self):
+        self._test_aggregation_buffering_simultaneously(socket.SOCK_DGRAM)
+
+    def test_aggregation_buffering_simultaneously_stream(self):
+        self._test_aggregation_buffering_simultaneously(socket.SOCK_STREAM)
+
+    def _test_aggregation_buffering_simultaneously(self, socket_kind):
+        dogstatsd = DogStatsd(disable_buffering=False, disable_aggregation=False, telemetry_min_flush_interval=0)
+        fake_socket = FakeSocket(socket_kind=socket_kind)
+        dogstatsd.socket = fake_socket
+        for _ in range(10):
+            dogstatsd.increment(u'test.ÀggregÀtion_and_buffering')
+        self.assertIsNone(fake_socket.recv(no_wait=True))
+        dogstatsd.flush_aggregated_metrics()
+        dogstatsd.flush()
+        self.assert_equal_telemetry(u'test.ÀggregÀtion_and_buffering:10|c\n', fake_socket.recv(2))
+
+    def test_aggregation_buffering_simultaneously_with_interval_dgram(self):
+        self._test_aggregation_buffering_simultaneously_with_interval(socket.SOCK_DGRAM)
+
+    def test_aggregation_buffering_simultaneously_with_interval_stream(self):
+        self._test_aggregation_buffering_simultaneously_with_interval(socket.SOCK_STREAM)
+    
+    def _test_aggregation_buffering_simultaneously_with_interval(self, socket_kind):
+        dogstatsd = DogStatsd(disable_buffering=False, disable_aggregation=False, flush_interval=1, telemetry_min_flush_interval=0)
+        fake_socket = FakeSocket(socket_kind=socket_kind)
+        dogstatsd.socket = fake_socket
+        for _ in range(10):
+            dogstatsd.increment('test.aggregation_and_buffering_with_interval')
+        self.assertIsNone(fake_socket.recv(no_wait=True))
+
+        time.sleep(0.3)
+        self.assertIsNone(fake_socket.recv(no_wait=True))
+
+        time.sleep(1)
+        self.assert_equal_telemetry(
+            'test.aggregation_and_buffering_with_interval:10|c\n',
             fake_socket.recv(2, no_wait=True)
         )
 
@@ -1016,7 +1228,7 @@ async def print_foo():
         dogstatsd.increment('page.views')
         self.assertIsNone(fake_socket.recv(no_wait=True))
 
-        time.sleep(DEFAULT_FLUSH_INTERVAL)
+        time.sleep(DEFAULT_BUFFERING_FLUSH_INTERVAL)
         self.assertIsNone(fake_socket.recv(no_wait=True))
 
         time.sleep(0.3)
@@ -1051,7 +1263,6 @@ async def print_foo():
             self.recv(2),
             telemetry=expected_metrics1)
 
-
         expected2 = 'page.views:123|g\ntimer:123|ms\n'
         self.assert_equal_telemetry(
             expected2,
@@ -1063,12 +1274,18 @@ async def print_foo():
             )
         )
 
-    def test_batching_runtime_changes(self):
+    def test_batching_runtime_changes_dgram(self):
+        self._test_batching_runtime_changes(socket.SOCK_DGRAM)
+
+    def test_batching_runtime_changes_stream(self):
+        self._test_batching_runtime_changes(socket.SOCK_STREAM)
+
+    def _test_batching_runtime_changes(self, socket_kind):
         dogstatsd = DogStatsd(
             disable_buffering=True,
             telemetry_min_flush_interval=0
         )
-        dogstatsd.socket = FakeSocket()
+        dogstatsd.socket = FakeSocket(socket_kind=socket_kind)
 
         # Send some unbuffered metrics and verify we got it immediately
         last_telemetry_size = self.send_and_assert(
@@ -1215,9 +1432,11 @@ async def print_foo():
         self.statsd.events_count = 2
         self.statsd.service_checks_count = 3
         self.statsd.bytes_sent = 4
-        self.statsd.bytes_dropped = 5
+        self.statsd.bytes_dropped_writer = 5
         self.statsd.packets_sent = 6
-        self.statsd.packets_dropped = 7
+        self.statsd.packets_dropped_writer = 7
+        self.statsd.bytes_dropped_queue = 8
+        self.statsd.packets_dropped_queue = 9
 
         self.statsd.open_buffer()
         self.statsd.gauge('page.views', 123)
@@ -1225,7 +1444,7 @@ async def print_foo():
 
         payload = 'page.views:123|g\n'
         telemetry = telemetry_metrics(metrics=2, events=2, service_checks=3, bytes_sent=4 + len(payload),
-                                      bytes_dropped=5, packets_sent=7, packets_dropped=7)
+                                      bytes_dropped_writer=5, packets_sent=7, packets_dropped_writer=7, bytes_dropped_queue=8, packets_dropped_queue=9)
 
         self.assert_equal_telemetry(payload, self.recv(2), telemetry=telemetry)
 
@@ -1233,9 +1452,11 @@ async def print_foo():
         self.assertEqual(0, self.statsd.events_count)
         self.assertEqual(0, self.statsd.service_checks_count)
         self.assertEqual(len(telemetry), self.statsd.bytes_sent)
-        self.assertEqual(0, self.statsd.bytes_dropped)
+        self.assertEqual(0, self.statsd.bytes_dropped_writer)
         self.assertEqual(1, self.statsd.packets_sent)
-        self.assertEqual(0, self.statsd.packets_dropped)
+        self.assertEqual(0, self.statsd.packets_dropped_writer)
+        self.assertEqual(0, self.statsd.bytes_dropped_queue)
+        self.assertEqual(0, self.statsd.packets_dropped_queue)
 
     def test_telemetry_flush_interval(self):
         dogstatsd = DogStatsd(disable_buffering=False)
@@ -1315,7 +1536,6 @@ async def print_foo():
         time1 = time.time()
         # setting the last flush time in the past to trigger a telemetry flush
         dogstatsd._last_flush_time = time1 - statsd._telemetry_flush_interval -1
-
         dogstatsd.close_buffer()
 
         metric = 'gauge1:1|g\ngauge2:2|g\n'
@@ -1323,7 +1543,6 @@ async def print_foo():
         # assert that _last_flush_time has been updated
         self.assertTrue(time1 < dogstatsd._last_flush_time)
 
-
     def test_dedicated_udp_telemetry_dest(self):
         listener_sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
         listener_sock.bind(('localhost', 0))
@@ -1353,6 +1572,35 @@ async def print_foo():
             expected_telemetry = telemetry_metrics(metrics=1, packets_sent=1, bytes_sent=8)
             self.assertEqual(udp_thread_telemetry_data, expected_telemetry)
 
+    def test_dedicated_udp6_telemetry_dest(self):
+        listener_sock = socket.socket(socket.AF_INET6, socket.SOCK_DGRAM)
+        listener_sock.bind(('localhost', 0))
+
+        def wait_for_data():
+            global udp_thread_telemetry_data
+            udp_thread_telemetry_data = listener_sock.recvfrom(UDP_OPTIMAL_PAYLOAD_LENGTH)[0].decode('utf-8')
+
+        with closing(listener_sock):
+            port = listener_sock.getsockname()[1]
+
+            dogstatsd = DogStatsd(
+                host="localhost",
+                port=12345,
+                telemetry_min_flush_interval=0,
+                telemetry_host="::1", # use explicit address, localhost may resolve to v4.
+                telemetry_port=port,
+            )
+
+            server = threading.Thread(target=wait_for_data)
+            server.start()
+
+            dogstatsd.increment('abc')
+
+            server.join(3)
+
+            expected_telemetry = telemetry_metrics(metrics=1, packets_sent=1, bytes_sent=8)
+            self.assertEqual(udp_thread_telemetry_data, expected_telemetry)
+
     def test_dedicated_uds_telemetry_dest(self):
         tempdir = tempfile.mkdtemp()
         socket_path = os.path.join(tempdir, 'socket.sock')
@@ -1406,6 +1654,67 @@ async def print_foo():
         )
         self.assertEqual(metrics_packet, fake_socket.recv(no_wait=True))
 
+    def test_context_manager_restores_enabled_buffering_state(self):
+        fake_socket = FakeSocket()
+        dogstatsd = DogStatsd(telemetry_min_flush_interval=0, disable_buffering=False)
+        dogstatsd.socket = fake_socket
+
+        with dogstatsd:
+            dogstatsd.gauge('page.views', 123)
+            dogstatsd.timing('timer', 123)
+
+        dogstatsd.gauge('newpage.views', 123)
+        dogstatsd.timing('newtimer', 123)
+
+        metric1 = "page.views:123|g"
+        metric2 = "timer:123|ms"
+        metric3 = "newpage.views:123|g"
+        metric4 = "newtimer:123|ms"
+
+        metrics1 = '\n'.join([metric1, metric2]) + "\n"
+        self.assertEqual(metrics1, fake_socket.recv(no_wait=True))
+
+        metrics_packet1 = telemetry_metrics(metrics=2, bytes_sent=len(metrics1), packets_sent=1)
+        self.assertEqual(metrics_packet1, fake_socket.recv(no_wait=True))
+
+        metrics2 = '\n'.join([metric3, metric4]) + "\n"
+        metrics_packet2 = telemetry_metrics(metrics=2, bytes_sent=len(metrics_packet1 + metrics2), packets_sent=2)
+        self.assertEqual(metrics2, fake_socket.recv(reset_wait=True))
+        self.assertEqual(metrics_packet2, fake_socket.recv())
+
+    def test_context_manager_restores_disabled_buffering_state(self):
+        fake_socket = FakeSocket()
+        dogstatsd = DogStatsd(telemetry_min_flush_interval=0, disable_buffering=True)
+        dogstatsd.socket = fake_socket
+
+        with dogstatsd:
+            dogstatsd.gauge('page.views', 123)
+            dogstatsd.timing('timer', 123)
+
+        dogstatsd.gauge('newpage.views', 123)
+        dogstatsd.timing('newtimer', 123)
+
+        metric1 = "page.views:123|g"
+        metric2 = "timer:123|ms"
+        metric3 = "newpage.views:123|g"
+        metric4 = "newtimer:123|ms"
+
+        metrics1 = '\n'.join([metric1, metric2]) + "\n"
+        self.assertEqual(metrics1, fake_socket.recv(no_wait=True))
+
+        metrics_packet1 = telemetry_metrics(metrics=2, bytes_sent=len(metrics1), packets_sent=1)
+        self.assertEqual(metrics_packet1, fake_socket.recv(no_wait=True))
+
+        metrics2 = '\n'.join([metric3]) + "\n"
+        metrics_packet2 = telemetry_metrics(metrics=1, bytes_sent=len(metrics_packet1 + metrics2), packets_sent=2)
+        self.assertEqual(metrics2, fake_socket.recv())
+        self.assertEqual(metrics_packet2, fake_socket.recv(no_wait=True))
+
+        metrics3 = '\n'.join([metric4]) + "\n"
+        metrics_packet3 = telemetry_metrics(metrics=1, bytes_sent=len(metrics_packet2 + metrics3), packets_sent=2)
+        self.assertEqual(metrics3, fake_socket.recv())
+        self.assertEqual(metrics_packet3, fake_socket.recv(no_wait=True))
+
     def test_batched_buffer_autoflush(self):
         fake_socket = FakeSocket()
         bytes_sent = 0
@@ -1475,6 +1784,35 @@ async def print_foo():
         self.assertEqual(metric, dogstatsd.socket.recv())
         self.assertEqual(telemetry_metrics(tags=tags, bytes_sent=len(metric)), dogstatsd.socket.recv())
 
+    def test_entity_id_and_container_id(self):
+        with preserve_environment_variable('DD_ENTITY_ID'):
+            os.environ['DD_ENTITY_ID'] = '04652bb7-19b7-11e9-9cc6-42010a9c016d'
+            dogstatsd = DogStatsd(telemetry_min_flush_interval=0)
+        dogstatsd.socket = FakeSocket()
+        dogstatsd._container_id = "ci-fake-container-id"
+
+        dogstatsd.increment("page.views")
+        dogstatsd.flush()
+        tags = "dd.internal.entity_id:04652bb7-19b7-11e9-9cc6-42010a9c016d"
+        metric = 'page.views:1|c|#' + tags + '|c:ci-fake-container-id\n'
+        self.assertEqual(metric, dogstatsd.socket.recv())
+        self.assertEqual(telemetry_metrics(tags=tags, bytes_sent=len(metric)), dogstatsd.socket.recv())
+
+    def test_entity_id_and_container_id_and_external_env(self):
+        with preserve_environment_variable('DD_ENTITY_ID'), preserve_environment_variable('DD_EXTERNAL_ENV'):
+            os.environ['DD_ENTITY_ID'] = '04652bb7-19b7-11e9-9cc6-42010a9c016d'
+            os.environ['DD_EXTERNAL_ENV'] = 'it-false,cn-container-name,pu-04652bb7-19b7-11e9-9cc6-42010a9c016d'
+            dogstatsd = DogStatsd(telemetry_min_flush_interval=0)
+        dogstatsd.socket = FakeSocket()
+        dogstatsd._container_id = "ci-fake-container-id"
+
+        dogstatsd.increment("page.views")
+        dogstatsd.flush()
+        tags = "dd.internal.entity_id:04652bb7-19b7-11e9-9cc6-42010a9c016d"
+        metric = 'page.views:1|c|#' + tags + '|c:ci-fake-container-id' + '|e:it-false,cn-container-name,pu-04652bb7-19b7-11e9-9cc6-42010a9c016d' + '\n'
+        self.assertEqual(metric, dogstatsd.socket.recv())
+        self.assertEqual(telemetry_metrics(tags=tags, bytes_sent=len(metric)), dogstatsd.socket.recv())
+
     def test_entity_tag_from_environment(self):
         with preserve_environment_variable('DD_ENTITY_ID'):
             os.environ['DD_ENTITY_ID'] = '04652bb7-19b7-11e9-9cc6-42010a9c016d'
@@ -1605,7 +1943,7 @@ async def print_foo():
             flush_interval=10000,
             disable_telemetry=True,
         )
-        dogstatsd.socket = FakeSocket()
+        dogstatsd.socket = FakeSocket(socket_path=dogstatsd.socket_path)
 
         for _ in range(10000):
             dogstatsd.increment('val')
@@ -1659,3 +1997,198 @@ async def print_foo():
     def test_histogram_does_not_send_none(self):
         self.statsd.histogram('metric', None)
         self.assertIsNone(self.recv())
+
+    def test_set_with_container_field(self):
+        self.statsd._container_id = "ci-fake-container-id"
+        self.statsd.set("set", 123)
+        self.assert_equal_telemetry("set:123|s|c:ci-fake-container-id\n", self.recv(2))
+        self.statsd._container_id = None
+
+    def test_gauge_with_container_field(self):
+        self.statsd._container_id = "ci-fake-container-id"
+        self.statsd.gauge("gauge", 123.4)
+        self.assert_equal_telemetry("gauge:123.4|g|c:ci-fake-container-id\n", self.recv(2))
+        self.statsd._container_id = None
+
+    def test_counter_with_container_field(self):
+        self.statsd._container_id = "ci-fake-container-id"
+
+        self.statsd.increment("page.views")
+        self.statsd.flush()
+        self.assert_equal_telemetry("page.views:1|c|c:ci-fake-container-id\n", self.recv(2))
+
+        self.statsd._reset_telemetry()
+        self.statsd.increment("page.views", 11)
+        self.statsd.flush()
+        self.assert_equal_telemetry("page.views:11|c|c:ci-fake-container-id\n", self.recv(2))
+
+        self.statsd._reset_telemetry()
+        self.statsd.decrement("page.views")
+        self.statsd.flush()
+        self.assert_equal_telemetry("page.views:-1|c|c:ci-fake-container-id\n", self.recv(2))
+
+        self.statsd._reset_telemetry()
+        self.statsd.decrement("page.views", 12)
+        self.statsd.flush()
+        self.assert_equal_telemetry("page.views:-12|c|c:ci-fake-container-id\n", self.recv(2))
+
+        self.statsd._container_id = None
+
+    def test_histogram_with_container_field(self):
+        self.statsd._container_id = "ci-fake-container-id"
+        self.statsd.histogram("histo", 123.4)
+        self.assert_equal_telemetry("histo:123.4|h|c:ci-fake-container-id\n", self.recv(2))
+        self.statsd._container_id = None
+
+    def test_timing_with_container_field(self):
+        self.statsd._container_id = "ci-fake-container-id"
+        self.statsd.timing("t", 123)
+        self.assert_equal_telemetry("t:123|ms|c:ci-fake-container-id\n", self.recv(2))
+        self.statsd._container_id = None
+
+    def test_event_with_container_field(self):
+        self.statsd._container_id = "ci-fake-container-id"
+        self.statsd.event(
+            "Title",
+            "L1\nL2",
+            priority="low",
+            date_happened=1375296969,
+        )
+        event2 = u"_e{5,6}:Title|L1\\nL2|d:1375296969|p:low|c:ci-fake-container-id\n"
+        self.assert_equal_telemetry(
+            event2,
+            self.recv(2),
+            telemetry=telemetry_metrics(
+                metrics=0,
+                events=1,
+                bytes_sent=len(event2),
+            ),
+        )
+
+        self.statsd._reset_telemetry()
+
+        self.statsd.event("Title", u"♬ †øU †øU ¥ºu T0µ ♪", aggregation_key="key", tags=["t1", "t2:v2"])
+        event3 = u"_e{5,32}:Title|♬ †øU †øU ¥ºu T0µ ♪|k:key|#t1,t2:v2|c:ci-fake-container-id\n"
+        self.assert_equal_telemetry(
+            event3,
+            self.recv(2, reset_wait=True),
+            telemetry=telemetry_metrics(
+                metrics=0,
+                events=1,
+                bytes_sent=len(event3),
+            ),
+        )
+        self.statsd._container_id = None
+
+    def test_service_check_with_container_field(self):
+        self.statsd._container_id = "ci-fake-container-id"
+        now = int(time.time())
+        self.statsd.service_check(
+            "my_check.name",
+            self.statsd.WARNING,
+            tags=["key1:val1", "key2:val2"],
+            timestamp=now,
+            hostname=u"i-abcd1234",
+            message=u"♬ †øU \n†øU ¥ºu|m: T0µ ♪",
+        )
+        check = u'_sc|my_check.name|{0}|d:{1}|h:i-abcd1234|#key1:val1,key2:val2|m:{2}|c:ci-fake-container-id\n'.format(
+            self.statsd.WARNING, now, u'♬ †øU \\n†øU ¥ºu|m\\: T0µ ♪'
+        )
+        self.assert_equal_telemetry(
+            check,
+            self.recv(2),
+            telemetry=telemetry_metrics(
+                metrics=0,
+                service_checks=1,
+                bytes_sent=len(check),
+            ),
+        )
+        self.statsd._container_id = None
+
+    def test_sender_mode(self):
+        statsd = DogStatsd(disable_background_sender=True)
+        self.assertIsNone(statsd._queue)
+
+        statsd.enable_background_sender()
+        self.assertIsNotNone(statsd._queue)
+
+        statsd = DogStatsd(disable_background_sender=False)
+        self.assertIsNotNone(statsd._queue)
+
+    def test_sender_calls_task_done(self):
+        statsd = DogStatsd(disable_background_sender=False)
+        statsd.socket = OverflownSocket()
+        statsd.increment("test.metric")
+        statsd.wait_for_pending()
+
+    def test_sender_queue_no_timeout(self):
+        statsd = DogStatsd(disable_background_sender=False, sender_queue_timeout=None)
+
+    def test_set_socket_timeout(self):
+        statsd = DogStatsd(disable_background_sender=False)
+        statsd.socket = FakeSocket()
+        statsd.set_socket_timeout(1)
+        self.assertEqual(statsd.socket.timeout, 1)
+        self.assertEqual(statsd.socket_timeout, 1)
+
+    def test_telemetry_api(self):
+        statsd = DogStatsd(disable_background_sender=False)
+
+        self.assertEqual(statsd.metrics_count, 0)
+        self.assertEqual(statsd.events_count, 0)
+        self.assertEqual(statsd.service_checks_count, 0)
+        self.assertEqual(statsd.bytes_sent, 0)
+        self.assertEqual(statsd.bytes_dropped, 0)
+        self.assertEqual(statsd.bytes_dropped_queue, 0)
+        self.assertEqual(statsd.bytes_dropped_writer, 0)
+        self.assertEqual(statsd.packets_sent, 0)
+        self.assertEqual(statsd.packets_dropped, 0)
+        self.assertEqual(statsd.packets_dropped_queue, 0)
+        self.assertEqual(statsd.packets_dropped_writer, 0)
+
+    def test_max_payload_size(self):
+        statsd = DogStatsd(socket_path=None, port=8125)
+        self.assertEqual(statsd._max_payload_size, UDP_OPTIMAL_PAYLOAD_LENGTH)
+
+        test_socket = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
+        statsd.socket = test_socket
+        self.assertEqual(statsd._max_payload_size, UDP_OPTIMAL_PAYLOAD_LENGTH)
+
+        test_socket = socket.socket(socket.AF_UNIX, socket.SOCK_DGRAM)
+        statsd.socket = test_socket
+        self.assertEqual(statsd._max_payload_size, UDS_OPTIMAL_PAYLOAD_LENGTH)
+
+    def test_post_fork_locks(self):
+        def inner():
+            statsd = DogStatsd(socket_path=None, port=8125)
+            # Statsd should survive this sequence of events
+            statsd.pre_fork()
+            statsd.get_socket()
+            statsd.post_fork_parent()
+        t = Thread(target=inner)
+        t.daemon = True
+        t.start()
+        t.join(timeout=5)
+        self.assertFalse(t.is_alive())
+
+    def test_fake_sockets(self):
+        """
+        To support legacy behavior wherein customers were able to set sockets directly as long as they supported a .send interface, 
+        ensure that arbitrary values passed to these properties are allowed and are handled correctly
+        """
+        statsd = DogStatsd(disable_buffering=True)
+
+        class fakeSock:
+            def __init__(self, id):
+                self.id = id
+            def send(self, _):
+                pass
+        statsd.socket = fakeSock(5)
+        statsd.telemetry_socket = fakeSock(10)
+
+        assert statsd.socket.id == 5
+        assert statsd.telemetry_socket.id == 10
+
+        statsd.increment("test", 1)
+
+        assert statsd.socket is not None
